=begin
WSDL4R - XMLSchema element definition for WSDL.
Copyright (C) 2002 NAKAMURA Hiroshi.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PRATICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 675 Mass
Ave, Cambridge, MA 02139, USA.
=end


require 'wsdl/info'


module WSDL
  module XMLSchema


class Element < Info
  attr_reader :name	# required
  attr_reader :type	# required
  attr_reader :maxOccurs
  attr_reader :minOccurs

  def initialize
    super()
    @name = nil
    @type = nil
    @maxOccurs = 1
    @minOccurs = 1
  end
  
  ComplexTypeName = Name.new( XSD::Namespace, 'complexType' )
  def parseElement( element )
    case element
    when ComplexTypeName
      o = ComplexType.new
      @complexType = o
      o
    else
      raise WSDLParser::UnknownElementError.new(
	"Unknown element #{ element }." )
    end
  end

  NameAttrName = Name.new( nil, 'name' )
  TypeAttrName = Name.new( nil, 'type' )
  MaxOccursAttrName = Name.new( nil, 'maxOccurs' )
  MinOccursAttrName = Name.new( nil, 'minOccurs' )
  def parseAttr( attr, value )
    case attr
    when NameAttrName
      @name = value
    when TypeAttrName
      @type = if value.is_a?( Name )
	  value
	else
	  Name.new( XSD::Namespace, value )
	end
    when MaxOccursAttrName
      if parent.type == Content::TypeAll
	if value != '1'
	  raise WSDLParser::AttrConstraintError.new(
	    "Cannot parse #{ value } for #{ attr }." )
	end
	@maxOccurs = value
      elsif parent.type == Content::TypeSequence
	@maxOccurs = value
      else
	raise NotImplementedError.new
      end
    when MinOccursAttrName
      if parent.type == Content::TypeAll
	if [ '0', '1' ].includes?( value )
	  @minOccurs = value
	else
	  raise WSDLParser::AttrConstraintError.new(
	    "Cannot parse #{ value } for #{ attr }." )
	end
      elsif parent.type == Content::TypeSequence
	@maxOccurs = value
      else
	raise NotImplementedError.new
      end
    else
      raise WSDLParser::UnknownAttributeError.new( "Unknown attr #{ attr }." )
    end
  end
end

  end
end
